@@ -3,6 +3,7 @@ package main
 import (
 	"errors"
 	"fmt"
+	"os"
 	"slices"
 	"strings"
 
@@ -16,6 +17,7 @@ import (
 	"github.com/foxboron/sbctl/logging"
 	"github.com/foxboron/sbctl/lsm"
 	"github.com/foxboron/sbctl/stringset"
+	"github.com/landlock-lsm/go-landlock/landlock"
 	"github.com/spf13/afero"
 	"github.com/spf13/cobra"
 )
@@ -63,6 +65,15 @@ var (
 		RunE: func(cmd *cobra.Command, args []string) error {
 			state := cmd.Context().Value(stateDataKey{}).(*config.State)
 			if state.Config.Landlock {
+				if enrollKeysCmdOptions.Export.Value != "" {
+					wd, err := os.Getwd()
+					if err != nil {
+						return err
+					}
+					lsm.RestrictAdditionalPaths(
+						landlock.RWDirs(wd),
+					)
+				}
 				if err := lsm.Restrict(); err != nil {
 					return err
 				}
@@ -253,11 +264,12 @@ func KeySync(state *config.State, oems []string) error {
 }
 
 func RunEnrollKeys(state *config.State) error {
-	// SetupMode is not necessarily required on a partial enrollment
 	ok, err := state.Efivarfs.GetSetupMode()
-	if err != nil {
+	// EFI variables are missing in some CI / build environments and setup mode is not needed for exporting keys
+	if err != nil && enrollKeysCmdOptions.Export.Value == "" {
 		return err
 	}
+	// SetupMode is not necessarily required for a partial enrollment and not needed for exporting keys
 	if !ok && enrollKeysCmdOptions.Partial.Value == "" && enrollKeysCmdOptions.Export.Value == "" {
 		return ErrSetupModeDisabled
 	}
@@ -309,7 +321,7 @@ func RunEnrollKeys(state *config.State) error {
 			return err
 		}
 	}
-	if !enrollKeysCmdOptions.Force && !enrollKeysCmdOptions.TPMEventlogChecksums && !enrollKeysCmdOptions.MicrosoftKeys {
+	if !enrollKeysCmdOptions.Force && !enrollKeysCmdOptions.TPMEventlogChecksums && !enrollKeysCmdOptions.MicrosoftKeys && !enrollKeysCmdOptions.Append {
 		if err := sbctl.CheckEventlogOprom(state.Fs, systemEventlog); err != nil {
 			return err
 		}
